/*
 * Copyright 2004-2010 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.slim3.controller.validator;

import java.util.Map;

import org.slim3.util.ApplicationMessage;

/**
 * A validator for minimum length check.
 * 
 * @author higa
 * @since 1.0.0
 * 
 */
public class MinlengthValidator extends AbstractValidator {

    /**
     * The minimum length.
     */
    protected int minlength;

    /**
     * Constructor.
     * 
     * @param minlength
     *            the minimum length
     */
    public MinlengthValidator(int minlength) {
        this(minlength, null);
    }

    /**
     * Constructor.
     * 
     * @param minlength
     *            the minimum length
     * @param message
     *            the error message
     */
    public MinlengthValidator(int minlength, String message) {
        super(message);
        this.minlength = minlength;
    }

    public String validate(Map<String, Object> parameters, String name) {
        Object value = parameters.get(name);
        if (value == null || "".equals(value)) {
            return null;
        }
        try {
            String s = (String) value;
            if (s.length() >= minlength) {
                return null;
            }
        } catch (Throwable ignore) {
        }
        if (message != null) {
            return message;
        }
        return ApplicationMessage.get(
            getMessageKey(),
            getLabel(name),
            minlength);
    }

    @Override
    protected String getMessageKey() {
        return "validator.minlength";
    }
}